from pymemcache.client import base
from pymemcache import serde
class Globals:

    # Don t put this in init, that will break with setattr
    client = base.Client(('localhost', 11211), serde=serde.pickle_serde)

    def __setattr__(self, key, value):
        self.client.set(key, value)
            # Note the variable down locally
        object.__setattr__(self, key, 'in_db')

    def __getattribute__(self, key):
        # print(f'Key: {key}')
        try:
            # To make class itself callable
            if object.__getattribute__(self, key) == 'in_db':
                return self.client.get(key)
            else:
                return  object.__getattribute__(self, key)


        except AttributeError:
            # If a variable doesn't exist in the class yet, check if it is in DB first.
            # If so create it as  local variable too.
            object.__setattr__(self, 'RetrievalValueTest', self.client.get(key))
            if   self.RetrievalValueTest is not None:
                # print('added key')
                object.__setattr__(self, key, 'in_db')
                return  self.RetrievalValueTest
            else:
                print(f'ERROR: Key --> {key} <-- not found in Memcahced Globals!')
                raise AttributeError



from EasyGlobals import EasyGlobals
g = EasyGlobals.Globals()
g.test1 = 4

print(g.test1)

g.test2 = 'hello world'
g.test3 = {'dictkey1': g.test1, 'dictkey2': g.test2} #  Dict

print(g.test1)
print(g.test2)
print(g.test3)






from EasyGlobals import EasyGlobals
# ---------------------

#  Example of a full class:
class TestClass:
    def __init__(self):
        self.a = 4
        self.b = 'test'

#  Getting and setting values from nested objects directly in the Globals can be problematic as the values are pickled while they are uploaded.
# It's best to retrieve the object, modify it locally, and then store the entire object again
# For exmaple:
globals.testclass = TestClass()
print(globals.testclass.a)

#  Nested variable setting doesn't work (yet?), can probably be implemented if there's demand.
globals.testclass.a = 5
print(globals.testclass.a)

# Do this instead:
holder = globals.testclass
holder.a = 99
globals.testclass = holder
print(globals.testclass.a)



from EasyGlobals import EasyGlobals
import multiprocessing

def write_to_globals():
    g = EasyGlobals.Globals()
    for i in range(1_000):
        g.testvar = i

def retrieve_from_globals(process_id):
    g = EasyGlobals.Globals()
    for i in range(10):
        result = g.testvar
        print(f'Process {process_id}, read: {result}')


print('Start writing process')
write_process = multiprocessing.Process(target=write_to_globals)
write_process.start()

print('Start reading with 3 simultaneous processes')
processlist = []
for i in range(5):
    processlist.append(multiprocessing.Process(target=retrieve_from_globals, args=(i,)))
    processlist[i].start()

for process in processlist:
    process.join()
print('Done reading')
write_process.join()
print('Done writing')
